#pragma once

#include "draw_strategy_base.hpp"

#include <iostream>
#include <gz-util/string/to_string.hpp>

namespace gz {
    /**
     * @brief Print the object to stdout when draw() is called
     * @details
     *  When update() is called, to object is converted to string and stored in tAsString.
     *  When draw() is called, tAsString is passed to `std::cout`.
     */
    template<gz::ConvertibleToString T>
    class PrintDrawStrategy : public DrawStrategy<T> {
        public:
            void draw() override;
            void update(const T& t) override;
        private:
            std::string tAsString;
    };


	template<gz::ConvertibleToString T>
	void PrintDrawStrategy<T>::draw() {
        std::cout << tAsString << '\n';
	}

	template<gz::ConvertibleToString T>
	void PrintDrawStrategy<T>::update(const T& t) {
        tAsString = gz::toString(t);
	}
}  // namespace gz
