#pragma once

#include "draw_strategy_base.hpp"

#include "buffer_manager.hpp"
#include "texture_manager.hpp"

namespace gz::vlk {
	class Renderer2D;
    class TextureManager;
    /**
     * @brief Draw rectangles using the vlk::Renderer2D
     *
     * @todo free resources and rule of 5
     */
    class VulkanDS {
        public:
            VulkanDS(Renderer2D& renderer, BufferManager<Vertex2D, uint32_t>& bufferManager)
                : renderer(renderer), bufferManager(bufferManager) {};

			const std::vector<Vertex2D>& getVertices() const { return vertices; }
			const std::vector<uint32_t>& getIndices() const { return indices; }
			const BufferInfo& getVertexBufferInfo() const { return vertexBufferInfo; }
			const BufferInfo& getIndexBufferInfo() const { return indexBufferInfo; }

            /**
             * @brief Add an offset to all indices (useful when putting multiple shapes in the same vertex buffer)
             */
            void setIndexOffset(uint32_t offset);
            /**
             * @brief Normalize the vertices, so that (1, 1) is (width, height)
             */
            void normalizeVertices(float width, float height);
            inline bool buffersInitalized() const { return vertexBufferInfo.index != BUFFER_NOT_INITIALIZED; }
            virtual ~VulkanDS() {};
        protected:
            /**
             * @todo free resources
             */
            void updateBufferInfos();
            std::vector<Vertex2D> vertices;
            std::vector<uint32_t> indices;
            BufferInfo vertexBufferInfo;
            BufferInfo indexBufferInfo;
            std::reference_wrapper<Renderer2D> renderer;
            std::reference_wrapper<BufferManager<Vertex2D, uint32_t>> bufferManager;
    };


    class VulkanTexturedDS {
        public:
            VulkanTexturedDS(TextureManager& textureManager, Texture texture);
			inline const Texture& getTexture() const { return texture; }
			inline uint32_t getTexureAtlasIndex() const { return textureAtlasIndex; }
            virtual ~VulkanTexturedDS() {};
        protected:
            Texture texture;
            TextureAtlasIndex textureAtlasIndex;
            std::reference_wrapper<TextureManager> textureManager;
    };

}  // namespace gz::vlk
